#include "MIDIUSB.h"

int timer0, timer1;
int motor_bit0 = 4;
int motor_bit1 = 5;
int motor_bit2 = 6;
int motor_bit3 = 7;
int motor_phase = 0;
int motor_speed = 500;
int motor_mode = 1;
int motor_direction;
int LED_speed = 5000;

void setup() {
  Serial.begin(115200);
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(motor_bit0, OUTPUT);
  pinMode(motor_bit1, OUTPUT);
  pinMode(motor_bit2, OUTPUT);
  pinMode(motor_bit3, OUTPUT);
  timer0 = timer1 = 0;
  motor_go();
}

void controlChange(byte channel, byte control, byte value) {
  midiEventPacket_t event = {0x0B, 0xB0 | channel, control, value};
  MidiUSB.sendMIDI(event);
}

void loop() {
  midiEventPacket_t rx;
  rx = MidiUSB.read();
  if (rx.header != 0) {
    if (rx.byte1 == 0xB0){
      switch(rx.byte2){
        case 0:
          LED_speed = 100 + 10 * rx.byte3;
          break;
        case 1:
          motor_speed = 10 + 2 * rx.byte3;
          break;
        case 2:
          motor_mode = rx.byte3;
          break;
      }
    }
  }
  if (++timer0 > 101){
    timer0 = 0;
    builtin_led();
  }
  if (++timer1 > 37){
    timer1 = 0;
    motor_go();
  }
}

void motor_go(){
  static int timer;
  timer++;
  if (timer < motor_speed) return;
  timer = 0;
  switch(motor_mode){
    case 1:
      motor_direction = 1;
      motor_mode_1phase();
      break;
    case 2:
      motor_direction = 3;
      motor_mode_1phase();
      break;
    case 3:
      motor_direction = 1;
      motor_mode_2phase();
      break;
    case 4:
      motor_direction = 3;
      motor_mode_2phase();
      break;
    case 0:
      return;
  }
  motor_phase = (motor_phase + motor_direction) & 3;
  return;
}

void motor_mode_1phase(){
  switch(motor_phase){
    case 0:
      digitalWrite(motor_bit0, LOW);
      digitalWrite(motor_bit1, LOW);
      digitalWrite(motor_bit2, LOW);
      digitalWrite(motor_bit3, HIGH);
      break;
    case 1:
      digitalWrite(motor_bit0, LOW);
      digitalWrite(motor_bit1, HIGH);
      digitalWrite(motor_bit2, LOW);
      digitalWrite(motor_bit3, LOW);
      break;
    case 2:
      digitalWrite(motor_bit0, LOW);
      digitalWrite(motor_bit1, LOW);
      digitalWrite(motor_bit2, HIGH);
      digitalWrite(motor_bit3, LOW);
      break;
    case 3:
      digitalWrite(motor_bit0, HIGH);
      digitalWrite(motor_bit1, LOW);
      digitalWrite(motor_bit2, LOW);
      digitalWrite(motor_bit3, LOW);
      break;
  }
  return;
}

void motor_mode_2phase(){
  switch(motor_phase){
    case 0:
      digitalWrite(motor_bit0, LOW);
      digitalWrite(motor_bit1, HIGH);
      digitalWrite(motor_bit2, LOW);
      digitalWrite(motor_bit3, HIGH);
      break;
    case 1:
      digitalWrite(motor_bit0, LOW);
      digitalWrite(motor_bit1, HIGH);
      digitalWrite(motor_bit2, HIGH);
      digitalWrite(motor_bit3, LOW);
      break;
    case 2:
      digitalWrite(motor_bit0, HIGH);
      digitalWrite(motor_bit1, LOW);
      digitalWrite(motor_bit2, HIGH);
      digitalWrite(motor_bit3, LOW);
      break;
    case 3:
      digitalWrite(motor_bit0, HIGH);
      digitalWrite(motor_bit1, LOW);
      digitalWrite(motor_bit2, LOW);
      digitalWrite(motor_bit3, HIGH);
      break;   
  }
  return;
}

void builtin_led(){
  static int timer, flag;
  timer++;
  if (timer > LED_speed){
    timer = 0;
    flag = ++flag & 1;
    digitalWrite(LED_BUILTIN, flag);
  }
}

